@@ -404,8 +404,15 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
 
     const auto &attributes = current_cmd->GetAttributes();
     auto cmd_name = attributes->name;
-    auto cmd_flags = attributes->GenerateFlags(cmd_tokens);
 
+    int tokens = static_cast<int>(cmd_tokens.size());
+    if (!attributes->CheckArity(tokens)) {
+      if (is_multi_exec) multi_error_ = true;
+      Reply(redis::Error({Status::NotOK, "wrong number of arguments"}));
+      continue;
+    }
+
+    auto cmd_flags = attributes->GenerateFlags(cmd_tokens);
     if (GetNamespace().empty()) {
       if (!password.empty()) {
         if (!(cmd_flags & kCmdAuth)) {
@@ -438,13 +445,6 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
       continue;
     }
 
-    int tokens = static_cast<int>(cmd_tokens.size());
-    if (!attributes->CheckArity(tokens)) {
-      if (is_multi_exec) multi_error_ = true;
-      Reply(redis::Error({Status::NotOK, "wrong number of arguments"}));
-      continue;
-    }
-
     current_cmd->SetArgs(cmd_tokens);
     auto s = current_cmd->Parse();
     if (!s.IsOK()) {
@@ -459,6 +459,11 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
       continue;
     }
 
+    if ((cmd_flags & kCmdAdmin) && !IsAdmin()) {
+      Reply(redis::Error({Status::RedisExecErr, errAdminPermissionRequired}));
+      continue;
+    }
+
     if (config->cluster_enabled) {
       s = srv_->cluster->CanExecByMySelf(attributes, cmd_tokens, this);
       if (!s.IsOK()) {
@@ -498,6 +503,12 @@ void Connection::ExecuteCommands(std::deque<CommandTokens> *to_process_cmds) {
       continue;
     }
 
+    ScopeExit in_script_exit{[this] { in_script_ = false; }, false};
+    if (attributes->category == CommandCategory::Script || attributes->category == CommandCategory::Function) {
+      in_script_ = true;
+      in_script_exit.Enable();
+    }
+
     SetLastCmd(cmd_name);
     {
       std::optional<MultiLockGuard> guard;
