@@ -27,8 +27,10 @@
 #include <sys/statvfs.h>
 #include <sys/utsname.h>
 
+#include <algorithm>
 #include <atomic>
 #include <cstdint>
+#include <cstdlib>
 #include <functional>
 #include <iomanip>
 #include <jsoncons/json.hpp>
@@ -42,6 +44,7 @@
 #include "config/config.h"
 #include "fmt/format.h"
 #include "redis_connection.h"
+#include "rocksdb/version.h"
 #include "storage/compaction_checker.h"
 #include "storage/redis_db.h"
 #include "storage/scripting.h"
@@ -113,7 +116,6 @@ Server::Server(engine::Storage *storage, Config *config)
   AdjustOpenFilesLimit();
   slow_log_.SetMaxEntries(config->slowlog_max_len);
   perf_log_.SetMaxEntries(config->profiling_sample_record_max_len);
-  lua_ = lua::CreateState();
 }
 
 Server::~Server() {
@@ -134,8 +136,6 @@ Server::~Server() {
   }
   cleanupExitedWorkerThreads(true /* force */);
   CleanupExitedSlaves();
-
-  lua::DestroyState(lua_);
 }
 
 // Kvrocks threads list:
@@ -176,14 +176,16 @@ Status Server::Start() {
   }
 
   if (config_->cluster_enabled) {
+    // Create objects used for slot migration
+    slot_migrator = std::make_unique<SlotMigrator>(this);
+
     if (config_->persist_cluster_nodes_enabled) {
       auto s = cluster->LoadClusterNodes(config_->NodesFilePath());
       if (!s.IsOK()) {
         return s.Prefixed("failed to load cluster nodes info");
       }
     }
-    // Create objects used for slot migration
-    slot_migrator = std::make_unique<SlotMigrator>(this);
+
     auto s = slot_migrator->CreateMigrationThread();
     if (!s.IsOK()) {
       return s.Prefixed("failed to create migration thread");
@@ -855,8 +857,10 @@ void Server::cron() {
   }
 }
 
-void Server::GetRocksDBInfo(std::string *info) {
-  std::ostringstream string_stream;
+Server::InfoEntries Server::GetRocksDBInfo() {
+  InfoEntries entries;
+  if (is_loading_) return entries;
+
   rocksdb::DB *db = storage->GetDB();
 
   uint64_t memtable_sizes = 0, cur_memtable_sizes = 0, num_snapshots = 0, num_running_flushes = 0;
@@ -872,18 +876,15 @@ void Server::GetRocksDBInfo(std::string *info) {
   db->GetAggregatedIntProperty("rocksdb.compaction-pending", &compaction_pending);
   db->GetAggregatedIntProperty("rocksdb.num-live-versions", &num_live_versions);
 
-  string_stream << "# RocksDB\r\n";
-
   {
     // All column families share the same block cache, so it's good to count a single one.
     uint64_t block_cache_usage = 0;
     uint64_t block_cache_pinned_usage = 0;
     auto subkey_cf_handle = storage->GetCFHandle(ColumnFamilyID::PrimarySubkey);
     db->GetIntProperty(subkey_cf_handle, rocksdb::DB::Properties::kBlockCacheUsage, &block_cache_usage);
-    string_stream << "block_cache_usage:" << block_cache_usage << "\r\n";
+    entries.emplace_back("block_cache_usage", block_cache_usage);
     db->GetIntProperty(subkey_cf_handle, rocksdb::DB::Properties::kBlockCachePinnedUsage, &block_cache_pinned_usage);
-    string_stream << "block_cache_pinned_usage[" << subkey_cf_handle->GetName() << "]:" << block_cache_pinned_usage
-                  << "\r\n";
+    entries.emplace_back("block_cache_pinned_usage[" + subkey_cf_handle->GetName() + "]", block_cache_pinned_usage);
 
     // All column faimilies share the same property of the DB, so it's good to count a single one.
     db->GetIntProperty(subkey_cf_handle, rocksdb::DB::Properties::kNumSnapshots, &num_snapshots);
@@ -896,27 +897,26 @@ void Server::GetRocksDBInfo(std::string *info) {
     uint64_t index_and_filter_cache_usage = 0;
     std::map<std::string, std::string> cf_stats_map;
     db->GetIntProperty(cf_handle, rocksdb::DB::Properties::kEstimateNumKeys, &estimate_keys);
-    string_stream << "estimate_keys[" << cf_handle->GetName() << "]:" << estimate_keys << "\r\n";
+    entries.emplace_back("estimate_keys[" + cf_handle->GetName() + "]", estimate_keys);
     db->GetIntProperty(cf_handle, rocksdb::DB::Properties::kEstimateTableReadersMem, &index_and_filter_cache_usage);
-    string_stream << "index_and_filter_cache_usage[" << cf_handle->GetName() << "]:" << index_and_filter_cache_usage
-                  << "\r\n";
+    entries.emplace_back("index_and_filter_cache_usage[" + cf_handle->GetName() + "]", index_and_filter_cache_usage);
     db->GetMapProperty(cf_handle, rocksdb::DB::Properties::kCFStats, &cf_stats_map);
-    string_stream << "level0_file_limit_slowdown[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["l0-file-count-limit-delays"] << "\r\n";
-    string_stream << "level0_file_limit_stop[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["l0-file-count-limit-stops"] << "\r\n";
-    string_stream << "pending_compaction_bytes_slowdown[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["pending-compaction-bytes-delays"] << "\r\n";
-    string_stream << "pending_compaction_bytes_stop[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["pending-compaction-bytes-stops"] << "\r\n";
-    string_stream << "level0_file_limit_stop_with_ongoing_compaction[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["cf-l0-file-count-limit-stops-with-ongoing-compaction"] << "\r\n";
-    string_stream << "level0_file_limit_slowdown_with_ongoing_compaction[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["cf-l0-file-count-limit-delays-with-ongoing-compaction"] << "\r\n";
-    string_stream << "memtable_count_limit_slowdown[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["memtable-limit-delays"] << "\r\n";
-    string_stream << "memtable_count_limit_stop[" << cf_handle->GetName()
-                  << "]:" << cf_stats_map["memtable-limit-stops"] << "\r\n";
+    entries.emplace_back("level0_file_limit_slowdown[" + cf_handle->GetName() + "]",
+                         cf_stats_map["l0-file-count-limit-delays"]);
+    entries.emplace_back("level0_file_limit_stop[" + cf_handle->GetName() + "]",
+                         cf_stats_map["l0-file-count-limit-stops"]);
+    entries.emplace_back("pending_compaction_bytes_slowdown[" + cf_handle->GetName() + "]",
+                         cf_stats_map["pending-compaction-bytes-delays"]);
+    entries.emplace_back("pending_compaction_bytes_stop[" + cf_handle->GetName() + "]",
+                         cf_stats_map["pending-compaction-bytes-stops"]);
+    entries.emplace_back("level0_file_limit_stop_with_ongoing_compaction[" + cf_handle->GetName() + "]",
+                         cf_stats_map["cf-l0-file-count-limit-stops-with-ongoing-compaction"]);
+    entries.emplace_back("level0_file_limit_slowdown_with_ongoing_compaction[" + cf_handle->GetName() + "]",
+                         cf_stats_map["cf-l0-file-count-limit-delays-with-ongoing-compaction"]);
+    entries.emplace_back("memtable_count_limit_slowdown[" + cf_handle->GetName() + "]",
+                         cf_stats_map["memtable-limit-delays"]);
+    entries.emplace_back("memtable_count_limit_stop[" + cf_handle->GetName() + "]",
+                         cf_stats_map["memtable-limit-stops"]);
   }
 
   auto rocksdb_stats = storage->GetDB()->GetDBOptions().statistics;
@@ -932,41 +932,39 @@ void Server::GetRocksDBInfo(std::string *info) {
         {"block_cache_data_miss", rocksdb::Tickers::BLOCK_CACHE_DATA_MISS},
     };
     for (const auto &iter : block_cache_stats) {
-      string_stream << iter.first << ":" << rocksdb_stats->getTickerCount(iter.second) << "\r\n";
+      entries.emplace_back(iter.first, rocksdb_stats->getTickerCount(iter.second));
     }
   }
 
-  string_stream << "all_mem_tables:" << memtable_sizes << "\r\n";
-  string_stream << "cur_mem_tables:" << cur_memtable_sizes << "\r\n";
-  string_stream << "snapshots:" << num_snapshots << "\r\n";
-  string_stream << "num_immutable_tables:" << num_immutable_tables << "\r\n";
-  string_stream << "num_running_flushes:" << num_running_flushes << "\r\n";
-  string_stream << "memtable_flush_pending:" << memtable_flush_pending << "\r\n";
-  string_stream << "compaction_pending:" << compaction_pending << "\r\n";
-  string_stream << "num_running_compactions:" << num_running_compaction << "\r\n";
-  string_stream << "num_live_versions:" << num_live_versions << "\r\n";
-  string_stream << "num_super_version:" << num_super_version << "\r\n";
-  string_stream << "num_background_errors:" << num_background_errors << "\r\n";
+  entries.emplace_back("all_mem_tables", memtable_sizes);
+  entries.emplace_back("cur_mem_tables", cur_memtable_sizes);
+  entries.emplace_back("snapshots", num_snapshots);
+  entries.emplace_back("num_immutable_tables", num_immutable_tables);
+  entries.emplace_back("num_running_flushes", num_running_flushes);
+  entries.emplace_back("memtable_flush_pending", memtable_flush_pending);
+  entries.emplace_back("compaction_pending", compaction_pending);
+  entries.emplace_back("num_running_compactions", num_running_compaction);
+  entries.emplace_back("num_live_versions", num_live_versions);
+  entries.emplace_back("num_super_version", num_super_version);
+  entries.emplace_back("num_background_errors", num_background_errors);
   auto db_stats = storage->GetDBStats();
-  string_stream << "flush_count:" << db_stats->flush_count << "\r\n";
-  string_stream << "compaction_count:" << db_stats->compaction_count << "\r\n";
-  string_stream << "put_per_sec:" << stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_PUT) << "\r\n";
-  string_stream << "get_per_sec:"
-                << stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_GET) +
-                       stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_MULTIGET)
-                << "\r\n";
-  string_stream << "seek_per_sec:" << stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_SEEK) << "\r\n";
-  string_stream << "next_per_sec:" << stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_NEXT) << "\r\n";
-  string_stream << "prev_per_sec:" << stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_PREV) << "\r\n";
+  entries.emplace_back("flush_count", db_stats->flush_count.load());
+  entries.emplace_back("compaction_count", db_stats->compaction_count.load());
+  entries.emplace_back("put_per_sec", stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_PUT));
+  entries.emplace_back("get_per_sec", stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_GET) +
+                                          stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_MULTIGET));
+  entries.emplace_back("seek_per_sec", stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_SEEK));
+  entries.emplace_back("next_per_sec", stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_NEXT));
+  entries.emplace_back("prev_per_sec", stats.GetInstantaneousMetric(STATS_METRIC_ROCKSDB_PREV));
   db_job_mu_.lock();
-  string_stream << "is_bgsaving:" << (is_bgsave_in_progress_ ? "yes" : "no") << "\r\n";
-  string_stream << "is_compacting:" << (db_compacting_ ? "yes" : "no") << "\r\n";
+  entries.emplace_back("is_bgsaving", (is_bgsave_in_progress_ ? "yes" : "no"));
+  entries.emplace_back("is_compacting", (db_compacting_ ? "yes" : "no"));
   db_job_mu_.unlock();
 
-  *info = string_stream.str();
+  return entries;
 }
 
-void Server::GetServerInfo(std::string *info) {
+Server::InfoEntries Server::GetServerInfo() {
   static int call_uname = 1;
   static utsname name;
   if (call_uname) {
@@ -975,97 +973,107 @@ void Server::GetServerInfo(std::string *info) {
     call_uname = 0;
   }
 
-  std::ostringstream string_stream;
-  string_stream << "# Server\r\n";
-  string_stream << "version:" << VERSION << "\r\n";
-  string_stream << "kvrocks_version:" << VERSION << "\r\n";
-  string_stream << "redis_version:" << REDIS_VERSION << "\r\n";
-  string_stream << "git_sha1:" << GIT_COMMIT << "\r\n";
-  string_stream << "kvrocks_git_sha1:" << GIT_COMMIT << "\r\n";
-  string_stream << "redis_mode:" << (config_->cluster_enabled ? "cluster" : "standalone") << "\r\n";
-  string_stream << "kvrocks_mode:" << (config_->cluster_enabled ? "cluster" : "standalone") << "\r\n";
-  string_stream << "os:" << name.sysname << " " << name.release << " " << name.machine << "\r\n";
-#ifdef __GNUC__
-  string_stream << "gcc_version:" << __GNUC__ << "." << __GNUC_MINOR__ << "." << __GNUC_PATCHLEVEL__ << "\r\n";
+  Server::InfoEntries entries;
+  entries.emplace_back("version", VERSION);  // deprecated
+  entries.emplace_back("kvrocks_version", VERSION);
+  entries.emplace_back("redis_version", REDIS_VERSION);
+  entries.emplace_back("git_sha1", GIT_COMMIT);  // deprecated
+  entries.emplace_back("kvrocks_git_sha1", GIT_COMMIT);
+  entries.emplace_back("redis_mode", (config_->cluster_enabled ? "cluster" : "standalone"));
+  entries.emplace_back("kvrocks_mode", (config_->cluster_enabled ? "cluster" : "standalone"));
+  entries.emplace_back("os", fmt::format("{} {} {}", name.sysname, name.release, name.machine));
+#if defined(__GNUC__) && !defined(__clang__)
+  entries.emplace_back("gcc_version", fmt::format("{}.{}.{}", __GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__));
 #endif
 #ifdef __clang__
-  string_stream << "clang_version:" << __clang_major__ << "." << __clang_minor__ << "." << __clang_patchlevel__
-                << "\r\n";
+  entries.emplace_back("clang_version",
+                       fmt::format("{}.{}.{}", __clang_major__, __clang_minor__, __clang_patchlevel__));
 #endif
-  string_stream << "arch_bits:" << sizeof(void *) * 8 << "\r\n";
-  string_stream << "process_id:" << getpid() << "\r\n";
-  string_stream << "tcp_port:" << config_->port << "\r\n";
+  entries.emplace_back("rocksdb_version", fmt::format("{}.{}.{}", ROCKSDB_MAJOR, ROCKSDB_MINOR, ROCKSDB_PATCH));
+  entries.emplace_back("arch_bits", sizeof(void *) * 8);
+  entries.emplace_back("process_id", getpid());
+  entries.emplace_back("tcp_port", config_->port);
+  entries.emplace_back("server_time_usec", util::GetTimeStampUS());
   int64_t now_secs = util::GetTimeStamp<std::chrono::seconds>();
-  string_stream << "uptime_in_seconds:" << now_secs - start_time_secs_ << "\r\n";
-  string_stream << "uptime_in_days:" << (now_secs - start_time_secs_) / 86400 << "\r\n";
-  *info = string_stream.str();
+  entries.emplace_back("uptime_in_seconds", now_secs - start_time_secs_);
+  entries.emplace_back("uptime_in_days", (now_secs - start_time_secs_) / 86400);
+#ifdef __linux__
+  if (auto exec_path = realpath("/proc/self/exe", nullptr)) {
+    entries.emplace_back("executable", exec_path);
+    free(exec_path);  // NOLINT(cppcoreguidelines-no-malloc)
+  }
+#endif
+  entries.emplace_back("config_file", config_->ConfigFilePath());
+  return entries;
 }
 
-void Server::GetClientsInfo(std::string *info) {
-  std::ostringstream string_stream;
-  string_stream << "# Clients\r\n";
-  string_stream << "maxclients:" << config_->maxclients << "\r\n";
-  string_stream << "connected_clients:" << connected_clients_ << "\r\n";
-  string_stream << "monitor_clients:" << monitor_clients_ << "\r\n";
-  string_stream << "blocked_clients:" << blocked_clients_ << "\r\n";
-  *info = string_stream.str();
+Server::InfoEntries Server::GetClientsInfo() {
+  InfoEntries entries;
+  entries.emplace_back("maxclients", config_->maxclients);
+  entries.emplace_back("connected_clients", connected_clients_.load());
+  entries.emplace_back("monitor_clients", monitor_clients_.load());
+  entries.emplace_back("blocked_clients", blocked_clients_.load());
+  return entries;
 }
 
-void Server::GetMemoryInfo(std::string *info) {
+Server::InfoEntries Server::GetMemoryInfo() {
   int64_t rss = Stats::GetMemoryRSS();
-  int memory_lua = lua_gc(lua_, LUA_GCCOUNT, 0) * 1024;
+  int64_t memory_lua = 0;
+  for (auto &wt : worker_threads_) {
+    memory_lua += wt->GetWorker()->GetLuaMemorySize();
+  }
   std::string used_memory_rss_human = util::BytesToHuman(rss);
   std::string used_memory_lua_human = util::BytesToHuman(memory_lua);
 
-  std::ostringstream string_stream;
-  string_stream << "# Memory\r\n";
-  string_stream << "used_memory_rss:" << rss << "\r\n";
-  string_stream << "used_memory_rss_human:" << used_memory_rss_human << "\r\n";
-  string_stream << "used_memory_lua:" << memory_lua << "\r\n";
-  string_stream << "used_memory_lua_human:" << used_memory_lua_human << "\r\n";
-  string_stream << "used_memory_startup:" << memory_startup_use_.load(std::memory_order_relaxed) << "\r\n";
-  *info = string_stream.str();
+  InfoEntries entries;
+  entries.emplace_back("used_memory_rss", rss);
+  entries.emplace_back("used_memory_rss_human", used_memory_rss_human);
+  entries.emplace_back("used_memory_lua", memory_lua);
+  entries.emplace_back("used_memory_lua_human", used_memory_lua_human);
+  entries.emplace_back("used_memory_startup", memory_startup_use_.load(std::memory_order_relaxed));
+  return entries;
 }
 
-void Server::GetReplicationInfo(std::string *info) {
-  std::ostringstream string_stream;
-  string_stream << "# Replication\r\n";
-  string_stream << "role:" << (IsSlave() ? "slave" : "master") << "\r\n";
+Server::InfoEntries Server::GetReplicationInfo() {
+  InfoEntries entries;
+  if (is_loading_) return entries;
+
+  entries.emplace_back("role", (IsSlave() ? "slave" : "master"));
   if (IsSlave()) {
     int64_t now_secs = util::GetTimeStamp<std::chrono::seconds>();
-    string_stream << "master_host:" << master_host_ << "\r\n";
-    string_stream << "master_port:" << master_port_ << "\r\n";
+    entries.emplace_back("master_host", master_host_);
+    entries.emplace_back("master_port", master_port_);
     ReplState state = GetReplicationState();
-    string_stream << "master_link_status:" << (state == kReplConnected ? "up" : "down") << "\r\n";
-    string_stream << "master_sync_unrecoverable_error:" << (state == kReplError ? "yes" : "no") << "\r\n";
-    string_stream << "master_sync_in_progress:" << (state == kReplFetchMeta || state == kReplFetchSST) << "\r\n";
-    string_stream << "master_last_io_seconds_ago:" << now_secs - replication_thread_->LastIOTimeSecs() << "\r\n";
-    string_stream << "slave_repl_offset:" << storage->LatestSeqNumber() << "\r\n";
-    string_stream << "slave_priority:" << config_->slave_priority << "\r\n";
+    entries.emplace_back("master_link_status", (state == kReplConnected ? "up" : "down"));
+    entries.emplace_back("master_sync_unrecoverable_error", (state == kReplError ? "yes" : "no"));
+    entries.emplace_back("master_sync_in_progress", (state == kReplFetchMeta || state == kReplFetchSST));
+    entries.emplace_back("master_last_io_seconds_ago", now_secs - replication_thread_->LastIOTimeSecs());
+    entries.emplace_back("slave_repl_offset", storage->LatestSeqNumber());
+    entries.emplace_back("slave_priority", config_->slave_priority);
   }
 
   int idx = 0;
   rocksdb::SequenceNumber latest_seq = storage->LatestSeqNumber();
 
   slave_threads_mu_.lock();
-  string_stream << "connected_slaves:" << slave_threads_.size() << "\r\n";
+  entries.emplace_back("connected_slaves", slave_threads_.size());
   for (const auto &slave : slave_threads_) {
     if (slave->IsStopped()) continue;
 
-    string_stream << "slave" << std::to_string(idx) << ":";
-    string_stream << "ip=" << slave->GetConn()->GetAnnounceIP() << ",port=" << slave->GetConn()->GetAnnouncePort()
-                  << ",offset=" << slave->GetCurrentReplSeq() << ",lag=" << latest_seq - slave->GetCurrentReplSeq()
-                  << "\r\n";
+    entries.emplace_back("slave" + std::to_string(idx),
+                         fmt::format("ip={},port={},offset={},lag={}", slave->GetConn()->GetAnnounceIP(),
+                                     slave->GetConn()->GetAnnouncePort(), slave->GetCurrentReplSeq(),
+                                     latest_seq - slave->GetCurrentReplSeq()));
     ++idx;
   }
   slave_threads_mu_.unlock();
 
-  string_stream << "master_repl_offset:" << latest_seq << "\r\n";
+  entries.emplace_back("master_repl_offset", latest_seq);
 
-  *info = string_stream.str();
+  return entries;
 }
 
-void Server::GetRoleInfo(std::string *info) {
+std::string Server::GetRoleInfo() {
   if (IsSlave()) {
     std::vector<std::string> roles;
     roles.emplace_back("slave");
@@ -1081,7 +1089,7 @@ void Server::GetRoleInfo(std::string *info) {
       roles.emplace_back("connecting");
     }
     roles.emplace_back(std::to_string(storage->LatestSeqNumber()));
-    *info = redis::ArrayOfBulkStrings(roles);
+    return redis::ArrayOfBulkStrings(roles);
   } else {
     std::vector<std::string> list;
 
@@ -1101,12 +1109,14 @@ void Server::GetRoleInfo(std::string *info) {
     if (list.size() > 0) {
       multi_len = 3;
     }
-    info->append(redis::MultiLen(multi_len));
-    info->append(redis::BulkString("master"));
-    info->append(redis::BulkString(std::to_string(storage->LatestSeqNumber())));
+    std::string info;
+    info.append(redis::MultiLen(multi_len));
+    info.append(redis::BulkString("master"));
+    info.append(redis::BulkString(std::to_string(storage->LatestSeqNumber())));
     if (list.size() > 0) {
-      info->append(redis::Array(list));
+      info.append(redis::Array(list));
     }
+    return info;
   }
 }
 
@@ -1134,46 +1144,45 @@ int64_t Server::GetLastBgsaveTime() {
   return last_bgsave_timestamp_secs_ == -1 ? start_time_secs_ : last_bgsave_timestamp_secs_;
 }
 
-void Server::GetStatsInfo(std::string *info) {
-  std::ostringstream string_stream;
-  string_stream << "# Stats\r\n";
-  string_stream << "total_connections_received:" << total_clients_ << "\r\n";
-  string_stream << "total_commands_processed:" << stats.total_calls << "\r\n";
-  string_stream << "instantaneous_ops_per_sec:" << stats.GetInstantaneousMetric(STATS_METRIC_COMMAND) << "\r\n";
-  string_stream << "total_net_input_bytes:" << stats.in_bytes << "\r\n";
-  string_stream << "total_net_output_bytes:" << stats.out_bytes << "\r\n";
-  string_stream << "instantaneous_input_kbps:"
-                << static_cast<float>(stats.GetInstantaneousMetric(STATS_METRIC_NET_INPUT) / 1024) << "\r\n";
-  string_stream << "instantaneous_output_kbps:"
-                << static_cast<float>(stats.GetInstantaneousMetric(STATS_METRIC_NET_OUTPUT) / 1024) << "\r\n";
-  string_stream << "sync_full:" << stats.fullsync_count << "\r\n";
-  string_stream << "sync_partial_ok:" << stats.psync_ok_count << "\r\n";
-  string_stream << "sync_partial_err:" << stats.psync_err_count << "\r\n";
+Server::InfoEntries Server::GetStatsInfo() {
+  Server::InfoEntries entries;
+  entries.emplace_back("total_connections_received", total_clients_.load());
+  entries.emplace_back("total_commands_processed", stats.total_calls.load());
+  entries.emplace_back("instantaneous_ops_per_sec", stats.GetInstantaneousMetric(STATS_METRIC_COMMAND));
+  entries.emplace_back("total_net_input_bytes", stats.in_bytes.load());
+  entries.emplace_back("total_net_output_bytes", stats.out_bytes.load());
+  entries.emplace_back("instantaneous_input_kbps",
+                       static_cast<float>(stats.GetInstantaneousMetric(STATS_METRIC_NET_INPUT) / 1024));
+  entries.emplace_back("instantaneous_output_kbps",
+                       static_cast<float>(stats.GetInstantaneousMetric(STATS_METRIC_NET_OUTPUT) / 1024));
+  entries.emplace_back("sync_full", stats.fullsync_count.load());
+  entries.emplace_back("sync_partial_ok", stats.psync_ok_count.load());
+  entries.emplace_back("sync_partial_err", stats.psync_err_count.load());
 
   auto db_stats = storage->GetDBStats();
-  string_stream << "keyspace_hits:" << db_stats->keyspace_hits << "\r\n";
-  string_stream << "keyspace_misses:" << db_stats->keyspace_misses << "\r\n";
+  entries.emplace_back("keyspace_hits", db_stats->keyspace_hits.load());
+  entries.emplace_back("keyspace_misses", db_stats->keyspace_misses.load());
 
   {
     std::lock_guard<std::mutex> lg(pubsub_channels_mu_);
-    string_stream << "pubsub_channels:" << pubsub_channels_.size() << "\r\n";
-    string_stream << "pubsub_patterns:" << pubsub_patterns_.size() << "\r\n";
+    entries.emplace_back("pubsub_channels", pubsub_channels_.size());
+    entries.emplace_back("pubsub_patterns", pubsub_patterns_.size());
   }
 
-  *info = string_stream.str();
+  return entries;
 }
 
-void Server::GetCommandsStatsInfo(std::string *info) {
-  std::ostringstream string_stream;
-  string_stream << "# Commandstats\r\n";
+Server::InfoEntries Server::GetCommandsStatsInfo() {
+  InfoEntries entries;
 
   for (const auto &cmd_stat : stats.commands_stats) {
     auto calls = cmd_stat.second.calls.load();
     if (calls == 0) continue;
 
     auto latency = cmd_stat.second.latency.load();
-    string_stream << "cmdstat_" << cmd_stat.first << ":calls=" << calls << ",usec=" << latency
-                  << ",usec_per_call=" << static_cast<float>(latency / calls) << "\r\n";
+    entries.emplace_back("cmdstat_" + cmd_stat.first,
+                         fmt::format("calls={},usec={},usec_per_call={}", calls, latency,
+                                     static_cast<double>(latency) / static_cast<double>(calls)));
   }
 
   for (const auto &cmd_hist : stats.commands_histogram) {
@@ -1182,7 +1191,7 @@ void Server::GetCommandsStatsInfo(std::string *info) {
     if (calls == 0) continue;
 
     auto sum = stats.commands_histogram[command_name].sum.load();
-    string_stream << "cmdstathist_" << command_name << ":";
+    std::string result;
     for (std::size_t i{0}; i < stats.commands_histogram[command_name].buckets.size(); ++i) {
       auto bucket_value = stats.commands_histogram[command_name].buckets[i]->load();
       auto bucket_bound = std::numeric_limits<double>::infinity();
@@ -1190,159 +1199,119 @@ void Server::GetCommandsStatsInfo(std::string *info) {
         bucket_bound = stats.bucket_boundaries[i];
       }
 
-      string_stream << bucket_bound << "=" << bucket_value << ",";
+      result.append(fmt::format("{}={},", bucket_bound, bucket_value));
     }
-    string_stream << "sum=" << sum << ",count=" << calls << "\r\n";
+    result.append(fmt::format("sum={},count={}", sum, calls));
+
+    entries.emplace_back("cmdstathist_" + command_name, result);
   }
 
-  *info = string_stream.str();
+  return entries;
 }
 
-void Server::GetClusterInfo(std::string *info) {
-  std::ostringstream string_stream;
+Server::InfoEntries Server::GetClusterInfo() {
+  InfoEntries entries;
 
-  string_stream << "# Cluster\r\n";
-  string_stream << "cluster_enabled:" << config_->cluster_enabled << "\r\n";
+  entries.emplace_back("cluster_enabled", config_->cluster_enabled);
 
-  *info = string_stream.str();
+  return entries;
+}
+
+Server::InfoEntries Server::GetPersistenceInfo() {
+  InfoEntries entries;
+
+  entries.emplace_back("loading", is_loading_.load());
+
+  std::lock_guard<std::mutex> lg(db_job_mu_);
+  entries.emplace_back("bgsave_in_progress", is_bgsave_in_progress_);
+  entries.emplace_back("last_bgsave_time",
+                       (last_bgsave_timestamp_secs_ == -1 ? start_time_secs_ : last_bgsave_timestamp_secs_));
+  entries.emplace_back("last_bgsave_status", last_bgsave_status_);
+  entries.emplace_back("last_bgsave_time_sec", last_bgsave_duration_secs_);
+
+  return entries;
+}
+
+Server::InfoEntries Server::GetCpuInfo() {  // NOLINT(readability-convert-member-functions-to-static)
+  InfoEntries entries;
+
+  rusage self_ru;
+  getrusage(RUSAGE_SELF, &self_ru);
+  entries.emplace_back("used_cpu_sys", static_cast<float>(self_ru.ru_stime.tv_sec) +
+                                           static_cast<float>(self_ru.ru_stime.tv_usec / 1000000));
+  entries.emplace_back("used_cpu_user", static_cast<float>(self_ru.ru_utime.tv_sec) +
+                                            static_cast<float>(self_ru.ru_utime.tv_usec / 1000000));
+
+  return entries;
+}
+
+Server::InfoEntries Server::GetKeyspaceInfo(const std::string &ns) {
+  InfoEntries entries;
+  if (is_loading_) return entries;
+
+  KeyNumStats stats;
+  GetLatestKeyNumStats(ns, &stats);
+  auto last_dbsize_scan_timestamp = static_cast<time_t>(GetLastScanTime(ns));
+
+  entries.emplace_back("last_dbsize_scan_timestamp", last_dbsize_scan_timestamp);
+  entries.emplace_back("db0", fmt::format("keys={},expires={},avg_ttl={},expired={}", stats.n_key, stats.n_expires,
+                                          stats.avg_ttl, stats.n_expired));
+  entries.emplace_back("sequence", storage->GetDB()->GetLatestSequenceNumber());
+  entries.emplace_back("used_db_size", storage->GetTotalSize(ns));
+  entries.emplace_back("max_db_size", config_->max_db_size * GiB);
+  double used_percent = config_->max_db_size ? static_cast<double>(storage->GetTotalSize() * 100) /
+                                                   static_cast<double>(config_->max_db_size * GiB)
+                                             : 0;
+  entries.emplace_back("used_percent", fmt::format("{}%", used_percent));
+
+  struct statvfs stat;
+  if (statvfs(config_->db_dir.c_str(), &stat) == 0) {
+    auto disk_capacity = stat.f_blocks * stat.f_frsize;
+    auto used_disk_size = (stat.f_blocks - stat.f_bavail) * stat.f_frsize;
+    entries.emplace_back("disk_capacity", disk_capacity);
+    entries.emplace_back("used_disk_size", used_disk_size);
+    double used_disk_percent = static_cast<double>(used_disk_size * 100) / static_cast<double>(disk_capacity);
+    entries.emplace_back("used_disk_percent", fmt::format("{}%", used_disk_percent));
+  }
+
+  return entries;
 }
 
 // WARNING: we must not access DB(i.e. RocksDB) when server is loading since
 // DB is closed and the pointer is invalid. Server may crash if we access DB during loading.
 // If you add new fields which access DB into INFO command output, make sure
 // this section can't be shown when loading(i.e. !is_loading_).
-void Server::GetInfo(const std::string &ns, const std::string &section, std::string *info) {
-  info->clear();
+std::string Server::GetInfo(const std::string &ns, const std::vector<std::string> &sections) {
+  std::vector<std::pair<std::string, std::function<InfoEntries(Server *)>>> info_funcs = {
+      {"Server", &Server::GetServerInfo},   {"Clients", &Server::GetClientsInfo},
+      {"Memory", &Server::GetMemoryInfo},   {"Persistence", &Server::GetPersistenceInfo},
+      {"Stats", &Server::GetStatsInfo},     {"Replication", &Server::GetReplicationInfo},
+      {"CPU", &Server::GetCpuInfo},         {"CommandStats", &Server::GetCommandsStatsInfo},
+      {"Cluster", &Server::GetClusterInfo}, {"Keyspace", [&ns](Server *srv) { return srv->GetKeyspaceInfo(ns); }},
+      {"RocksDB", &Server::GetRocksDBInfo},
+  };
 
-  std::ostringstream string_stream;
-  bool all = section == "all";
-  int section_cnt = 0;
+  std::string info_str;
 
-  if (all || section == "server") {
-    std::string server_info;
-    GetServerInfo(&server_info);
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << server_info;
-  }
+  bool all = sections.empty() || util::FindICase(sections.begin(), sections.end(), "all") != sections.end();
 
-  if (all || section == "clients") {
-    std::string clients_info;
-    GetClientsInfo(&clients_info);
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << clients_info;
-  }
+  bool first = true;
+  for (const auto &[sec, fn] : info_funcs) {
+    if (all || util::FindICase(sections.begin(), sections.end(), sec) != sections.end()) {
+      if (first)
+        first = false;
+      else
+        info_str.append("\r\n");
 
-  if (all || section == "memory") {
-    std::string memory_info;
-    GetMemoryInfo(&memory_info);
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << memory_info;
-  }
+      info_str.append("# " + sec + "\r\n");
 
-  if (all || section == "persistence") {
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << "# Persistence\r\n";
-    string_stream << "loading:" << is_loading_ << "\r\n";
-
-    std::lock_guard<std::mutex> lg(db_job_mu_);
-    string_stream << "bgsave_in_progress:" << (is_bgsave_in_progress_ ? 1 : 0) << "\r\n";
-    string_stream << "last_bgsave_time:"
-                  << (last_bgsave_timestamp_secs_ == -1 ? start_time_secs_ : last_bgsave_timestamp_secs_) << "\r\n";
-    string_stream << "last_bgsave_status:" << last_bgsave_status_ << "\r\n";
-    string_stream << "last_bgsave_time_sec:" << last_bgsave_duration_secs_ << "\r\n";
-  }
-
-  if (all || section == "stats") {
-    std::string stats_info;
-    GetStatsInfo(&stats_info);
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << stats_info;
-  }
-
-  // In replication section, we access DB, so we can't do that when loading
-  if (!is_loading_ && (all || section == "replication")) {
-    std::string replication_info;
-    GetReplicationInfo(&replication_info);
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << replication_info;
-  }
-
-  if (all || section == "cpu") {
-    rusage self_ru;
-    getrusage(RUSAGE_SELF, &self_ru);
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << "# CPU\r\n";
-    string_stream << "used_cpu_sys:"
-                  << static_cast<float>(self_ru.ru_stime.tv_sec) +
-                         static_cast<float>(self_ru.ru_stime.tv_usec / 1000000)
-                  << "\r\n";
-    string_stream << "used_cpu_user:"
-                  << static_cast<float>(self_ru.ru_utime.tv_sec) +
-                         static_cast<float>(self_ru.ru_utime.tv_usec / 1000000)
-                  << "\r\n";
-  }
-
-  if (all || section == "commandstats") {
-    std::string commands_stats_info;
-    GetCommandsStatsInfo(&commands_stats_info);
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << commands_stats_info;
-  }
-
-  if (all || section == "cluster") {
-    std::string cluster_info;
-    GetClusterInfo(&cluster_info);
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << cluster_info;
-  }
-
-  // In keyspace section, we access DB, so we can't do that when loading
-  if (!is_loading_ && (all || section == "keyspace")) {
-    KeyNumStats stats;
-    GetLatestKeyNumStats(ns, &stats);
-
-    // FIXME(mwish): output still requires std::tm.
-    auto last_scan_time = static_cast<time_t>(GetLastScanTime(ns));
-    std::tm last_scan_tm{};
-    localtime_r(&last_scan_time, &last_scan_tm);
-
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << "# Keyspace\r\n";
-    if (last_scan_time == 0) {
-      string_stream << "# WARN: DBSIZE SCAN never performed yet\r\n";
-    } else {
-      string_stream << "# Last DBSIZE SCAN time: " << std::put_time(&last_scan_tm, "%a %b %e %H:%M:%S %Y") << "\r\n";
-    }
-    string_stream << "db0:keys=" << stats.n_key << ",expires=" << stats.n_expires << ",avg_ttl=" << stats.avg_ttl
-                  << ",expired=" << stats.n_expired << "\r\n";
-    string_stream << "sequence:" << storage->GetDB()->GetLatestSequenceNumber() << "\r\n";
-    string_stream << "used_db_size:" << storage->GetTotalSize(ns) << "\r\n";
-    string_stream << "max_db_size:" << config_->max_db_size * GiB << "\r\n";
-    double used_percent = config_->max_db_size ? static_cast<double>(storage->GetTotalSize() * 100) /
-                                                     static_cast<double>(config_->max_db_size * GiB)
-                                               : 0;
-    string_stream << "used_percent: " << used_percent << "%\r\n";
-
-    struct statvfs stat;
-    if (statvfs(config_->db_dir.c_str(), &stat) == 0) {
-      auto disk_capacity = stat.f_blocks * stat.f_frsize;
-      auto used_disk_size = (stat.f_blocks - stat.f_bavail) * stat.f_frsize;
-      string_stream << "disk_capacity:" << disk_capacity << "\r\n";
-      string_stream << "used_disk_size:" << used_disk_size << "\r\n";
-      double used_disk_percent = static_cast<double>(used_disk_size * 100) / static_cast<double>(disk_capacity);
-      string_stream << "used_disk_percent: " << used_disk_percent << "%\r\n";
+      for (const auto &entry : fn(this)) {
+        info_str.append(fmt::format("{}:{}\r\n", entry.name, entry.val));
+      }
     }
   }
 
-  // In rocksdb section, we access DB, so we can't do that when loading
-  if (!is_loading_ && (all || section == "rocksdb")) {
-    std::string rocksdb_info;
-    GetRocksDBInfo(&rocksdb_info);
-    if (section_cnt++) string_stream << "\r\n";
-    string_stream << rocksdb_info;
-  }
-
-  *info = string_stream.str();
+  return info_str;
 }
 
 std::string Server::GetRocksDBStatsJson() const {
@@ -1733,11 +1702,7 @@ StatusOr<std::unique_ptr<redis::Commander>> Server::LookupAndCreateCommand(const
   return std::move(cmd);
 }
 
-Status Server::ScriptExists(const std::string &sha) {
-  if (lua::ScriptExists(lua_, sha)) {
-    return Status::OK();
-  }
-
+Status Server::ScriptExists(const std::string &sha) const {
   std::string body;
   return ScriptGet(sha, &body);
 }
@@ -1794,8 +1759,9 @@ Status Server::FunctionSetLib(const std::string &func, const std::string &lib) c
 }
 
 void Server::ScriptReset() {
-  auto lua = lua_.exchange(lua::CreateState());
-  lua::DestroyState(lua);
+  for (auto &wt : worker_threads_) {
+    wt->GetWorker()->LuaReset();
+  }
 }
 
 Status Server::ScriptFlush() {
