@@ -44,26 +44,26 @@ rocksdb::Status Disk::GetApproximateSizes(const Metadata &metadata, const Slice
   return rocksdb::Status::OK();
 }
 
-rocksdb::Status Disk::GetKeySize(const Slice &user_key, RedisType type, uint64_t *key_size) {
+rocksdb::Status Disk::GetKeySize(engine::Context &ctx, const Slice &user_key, RedisType type, uint64_t *key_size) {
   *key_size = 0;
   std::string ns_key = AppendNamespacePrefix(user_key);
   switch (type) {
     case RedisType::kRedisString:
       return GetStringSize(ns_key, key_size);
     case RedisType::kRedisHash:
-      return GetHashSize(ns_key, key_size);
+      return GetHashSize(ctx, ns_key, key_size);
     case RedisType::kRedisBitmap:
-      return GetBitmapSize(ns_key, key_size);
+      return GetBitmapSize(ctx, ns_key, key_size);
     case RedisType::kRedisList:
-      return GetListSize(ns_key, key_size);
+      return GetListSize(ctx, ns_key, key_size);
     case RedisType::kRedisSet:
-      return GetSetSize(ns_key, key_size);
+      return GetSetSize(ctx, ns_key, key_size);
     case RedisType::kRedisSortedint:
-      return GetSortedintSize(ns_key, key_size);
+      return GetSortedintSize(ctx, ns_key, key_size);
     case RedisType::kRedisZSet:
-      return GetZsetSize(ns_key, key_size);
+      return GetZsetSize(ctx, ns_key, key_size);
     case RedisType::kRedisStream:
-      return GetStreamSize(ns_key, key_size);
+      return GetStreamSize(ctx, ns_key, key_size);
     default:
       return rocksdb::Status::NotFound("Not found ", user_key);
   }
@@ -75,32 +75,32 @@ rocksdb::Status Disk::GetStringSize(const Slice &ns_key, uint64_t *key_size) {
   return storage_->GetDB()->GetApproximateSizes(option_, metadata_cf_handle_, &key_range, 1, key_size);
 }
 
-rocksdb::Status Disk::GetHashSize(const Slice &ns_key, uint64_t *key_size) {
+rocksdb::Status Disk::GetHashSize(engine::Context &ctx, const Slice &ns_key, uint64_t *key_size) {
   HashMetadata metadata(false);
-  rocksdb::Status s = Database::GetMetadata(Database::GetOptions{}, {kRedisHash}, ns_key, &metadata);
+  rocksdb::Status s = Database::GetMetadata(ctx, {kRedisHash}, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   return GetApproximateSizes(metadata, ns_key, storage_->GetCFHandle(ColumnFamilyID::PrimarySubkey), key_size);
 }
 
-rocksdb::Status Disk::GetSetSize(const Slice &ns_key, uint64_t *key_size) {
+rocksdb::Status Disk::GetSetSize(engine::Context &ctx, const Slice &ns_key, uint64_t *key_size) {
   SetMetadata metadata(false);
-  rocksdb::Status s = Database::GetMetadata(Database::GetOptions{}, {kRedisSet}, ns_key, &metadata);
+  rocksdb::Status s = Database::GetMetadata(ctx, {kRedisSet}, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   return GetApproximateSizes(metadata, ns_key, storage_->GetCFHandle(ColumnFamilyID::PrimarySubkey), key_size);
 }
 
-rocksdb::Status Disk::GetListSize(const Slice &ns_key, uint64_t *key_size) {
+rocksdb::Status Disk::GetListSize(engine::Context &ctx, const Slice &ns_key, uint64_t *key_size) {
   ListMetadata metadata(false);
-  rocksdb::Status s = Database::GetMetadata(Database::GetOptions{}, {kRedisList}, ns_key, &metadata);
+  rocksdb::Status s = Database::GetMetadata(ctx, {kRedisList}, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   std::string buf;
   PutFixed64(&buf, metadata.head);
   return GetApproximateSizes(metadata, ns_key, storage_->GetCFHandle(ColumnFamilyID::PrimarySubkey), key_size, buf);
 }
 
-rocksdb::Status Disk::GetZsetSize(const Slice &ns_key, uint64_t *key_size) {
+rocksdb::Status Disk::GetZsetSize(engine::Context &ctx, const Slice &ns_key, uint64_t *key_size) {
   ZSetMetadata metadata(false);
-  rocksdb::Status s = Database::GetMetadata(Database::GetOptions{}, {kRedisZSet}, ns_key, &metadata);
+  rocksdb::Status s = Database::GetMetadata(ctx, {kRedisZSet}, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   std::string score_bytes;
   PutDouble(&score_bytes, kMinScore);
@@ -110,17 +110,17 @@ rocksdb::Status Disk::GetZsetSize(const Slice &ns_key, uint64_t *key_size) {
   return GetApproximateSizes(metadata, ns_key, storage_->GetCFHandle(ColumnFamilyID::PrimarySubkey), key_size);
 }
 
-rocksdb::Status Disk::GetBitmapSize(const Slice &ns_key, uint64_t *key_size) {
+rocksdb::Status Disk::GetBitmapSize(engine::Context &ctx, const Slice &ns_key, uint64_t *key_size) {
   BitmapMetadata metadata(false);
-  rocksdb::Status s = Database::GetMetadata(Database::GetOptions{}, {kRedisBitmap}, ns_key, &metadata);
+  rocksdb::Status s = Database::GetMetadata(ctx, {kRedisBitmap}, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   return GetApproximateSizes(metadata, ns_key, storage_->GetCFHandle(ColumnFamilyID::PrimarySubkey), key_size,
                              std::to_string(0), std::to_string(0));
 }
 
-rocksdb::Status Disk::GetSortedintSize(const Slice &ns_key, uint64_t *key_size) {
+rocksdb::Status Disk::GetSortedintSize(engine::Context &ctx, const Slice &ns_key, uint64_t *key_size) {
   SortedintMetadata metadata(false);
-  rocksdb::Status s = Database::GetMetadata(Database::GetOptions{}, {kRedisSortedint}, ns_key, &metadata);
+  rocksdb::Status s = Database::GetMetadata(ctx, {kRedisSortedint}, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   std::string start_buf;
   PutFixed64(&start_buf, 0);
@@ -128,9 +128,9 @@ rocksdb::Status Disk::GetSortedintSize(const Slice &ns_key, uint64_t *key_size)
                              start_buf, start_buf);
 }
 
-rocksdb::Status Disk::GetStreamSize(const Slice &ns_key, uint64_t *key_size) {
+rocksdb::Status Disk::GetStreamSize(engine::Context &ctx, const Slice &ns_key, uint64_t *key_size) {
   StreamMetadata metadata(false);
-  rocksdb::Status s = Database::GetMetadata(Database::GetOptions{}, {kRedisStream}, ns_key, &metadata);
+  rocksdb::Status s = Database::GetMetadata(ctx, {kRedisStream}, ns_key, &metadata);
   if (!s.ok()) return s.IsNotFound() ? rocksdb::Status::OK() : s;
   return GetApproximateSizes(metadata, ns_key, storage_->GetCFHandle(ColumnFamilyID::Stream), key_size);
 }
