@@ -22,13 +22,16 @@
 
 namespace redis {
 
-rocksdb::Status PubSub::Publish(const Slice &channel, const Slice &value) {
+rocksdb::Status PubSub::Publish(engine::Context &ctx, const Slice &channel, const Slice &value) {
   if (storage_->GetConfig()->IsSlave()) {
     return rocksdb::Status::NotSupported("can't publish to db in slave mode");
   }
   auto batch = storage_->GetWriteBatchBase();
-  batch->Put(pubsub_cf_handle_, channel, value);
-  return storage_->Write(storage_->DefaultWriteOptions(), batch->GetWriteBatch());
+  auto s = batch->Put(pubsub_cf_handle_, channel, value);
+  if (!s.ok()) {
+    return s;
+  }
+  return storage_->Write(ctx, storage_->DefaultWriteOptions(), batch->GetWriteBatch());
 }
 
 }  // namespace redis
