@@ -75,8 +75,8 @@ Status FunctionCall(redis::Connection *conn, const std::string &name, const std:
 Status FunctionList(Server *srv, const redis::Connection *conn, const std::string &libname, bool with_code,
                     std::string *output);
 Status FunctionListFunc(Server *srv, const redis::Connection *conn, const std::string &funcname, std::string *output);
-Status FunctionListLib(Server *srv, const redis::Connection *conn, const std::string &libname, std::string *output);
-Status FunctionDelete(engine::Context &ctx, Server *srv, const std::string &name);
+Status FunctionListLib(redis::Connection *conn, const std::string &libname, std::string *output);
+Status FunctionDelete(engine::Context &ctx, redis::Connection *conn, const std::string &name);
 bool FunctionIsLibExist(redis::Connection *conn, const std::string &libname, bool need_check_storage = true,
                         bool read_only = false);
 
